SHORT_NAME = "short_name"
TITLE = "title"
PARAMETERS = "parameters"
NAME = "name"
FORMAT_TYPE = "format_type"
REQUIRED = "required"
DEFAULT_VALUE = "default_value"
LABEL = "label"
HELP_STRING = "help_string"
HELP_LINK = "help_link"
FORM_CLASS = "form_class"
FORM_STYLE = "form_style"
POSSIBLE_VALUES = "possible_values"
MODULAR_ALERTS = "modular_alerts"
PLACEHOLDER = "placeholder"
LINK_TEXT = "link_text"
LINK_URL = "link_url"
LINK_TIP = "link_tip"
LINK_TYPE = "link_type"
ICON_PATH = "icon_path"
ALERT_PROPS = "alert_props"
RESULTS = "results"
RESULT = "result"
RESULTS_FILE = "results_file"
STDIN_FIELDS = "stdin_fields"
LARGE_ICON = "largeIcon"
